%% TuringGM
% Full parameter model
% based on the Gierer-Meinhardt model in chapter 2 of Murray's book

% Model definition file
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = TuringGM_squ_SMRD(t, y, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity

    N = paramsSys.N; % number of gridpoints
    L = paramsSys.L;
    W = paramsSys.W;
    k = paramsModel.k;
    D = paramsModel.D;
  

%     c = interp1(paramsSys.tDyNois', paramsSys.yDyNois', t);

    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*N, 1);            

            %break out y into molecular species (they are interleaved)
            A = y(1 : numSpecies : end - 1);
            R = y(2 : numSpecies : end - 0);


            %equations for non-diffusible A and R
%             dAdt = a_A * c' .* ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A; 
            dAdt = k(1) - k(2) * A + k(3) * A.^2 ./ R;
            %equations for R depend on boundary conditions (due to many iterations of model writing, there
            %could be some mistakes in some b.c.'s)
%             dRdt = a_R * c' .* ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;% + 1 * rdcell;
            dRdt = k(4) * A.^2 - k(5) * R;
            if paramsModel.CutOff ~= 0
                scale = 10^(-paramsModel.CutOff); % scale ~ sensitivity
                Ir = sparse(diag(R));
                TrsfrM = sparse(paramsSys.Trans * Ir - Ir * paramsSys.Trans);
%                 TrsfrM(TrsfrM ~= 0) = TrsfrM(TrsfrM ~= 0) - 1./(TrsfrM(TrsfrM ~= 0)./paramsModel.CutOff  + 1./TrsfrM(TrsfrM ~= 0));
                TrsfrM(TrsfrM ~= 0) = TrsfrM(TrsfrM ~= 0) - 1./(TrsfrM(TrsfrM ~= 0)./scale  + 1./TrsfrM(TrsfrM ~= 0));
                dRdt = dRdt + D * sum(TrsfrM, 2);
            else
                dRdt_Df = (D * R' * paramsSys.Q)';
                dRdt = dRdt + dRdt_Df; 
            end

%             dAdt = dAdt .* (1 + 0.01 * paramsSys.TgrInts * 2 * (rand(paramsSys.N, 1)-0.5));

            %construct output dydt from dh and dp
            output(1 : numSpecies : end - 1) = dAdt;
            output(2 : numSpecies : end - 0) = dRdt;
 
  
            
        case 'preModel' %pre-equilibration model
            %break out y into molecular species (they are interleaved)
             % the fixed concentration of Hh to use for pre-equilibration
            A = y(1);
            R = y(2);

            dAdt = k(1) - k(2) * A + k(3) * A.^2 ./ R;
            dRdt = k(4) * A.^2 - k(5) * R;
            
            %output
            output = [dAdt; dRdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;

 
        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
            % everything EXCEPT integrated signal q
%             q = 5 : numSpecies : numSpecies*N;   
            temp = 1 : numSpecies*N;
            output = temp; %look at everything except q
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2];
            
        case 'preModelIC' %specified initial conditions for preModel
            A0 = (k(1)+1)/k(2);%a_p / c_p;
            R0 = A0^2;
            tspan = [0 10000];
            y0 = [A0 R0];
            [t, y] = ode15s(@(t, y)preSol(t, y, paramsModel), tspan, y0, paramsModel);
            output = [y(end,1), y(end,2)];
    end
end
function dydt = preSol(t, y, paramsModel)
    k = paramsModel.k;
    dydt = [k(1) - k(2) * y(1) + k(3) * y(1).^2 ./ y(2),
            k(4) * y(1).^2 - k(5) * y(2)];
end
