%%%%%%%% Kinetics_squ_Simu.m %%%%%%%%
% Edit from 0703/1_Simulation_platform/200715_FinalCode/Ori
% Simulation platform for the SMRD paper 2C2D/2C1D + RN/LA/SP + no noise
% 2C1D_LA_influx

% Full parameter model
% Model definition file
% Simulates model for N gridpoints, using a vector of N*numSpecies
%       E.g., for species A, R, and TD (numSpecies  == 3), and N == 3
%             y = [h1, p1, c1, r1, q1, h2, p2, c2, r2, q2, h3, p3, c3, r3, q3], where h1 =
%             concentration of h at cell 1, h2 = concentration of h at cell 2, etc
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = Kinetics_squ_Simu(t, y, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity
    %bdryType = paramsSys.bdryType;
    N = paramsSys.N; % number of gridpoints
    L = paramsSys.L;
    W = paramsSys.W;
    %rdcell = paramsSys.rdcell;
    %rdcell2 = rand(paramsSys.N, 1);
    a_A = paramsModel.a_A;
    a_R = paramsModel.a_R;
    
    g_A = paramsModel.g_A; 
    g_R = paramsModel.g_R; 
    
    K_A = paramsModel.K_A;
    K_R = paramsModel.K_R;
    
    K = paramsModel.K;    
    Dr = paramsModel.Dr;
    Da = paramsModel.Da;
    H = paramsModel.H;


    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*N, 1);            

            %break out y into molecular species (they are interleaved)
            A = y(1 : numSpecies : end - 1);
            R = y(2 : numSpecies : end - 0);


            %equations for non-diffusible A and R
            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;% + 0.1 * rdcell; 

            %equations for R depend on boundary conditions (due to many iterations of model writing, there
            %could be some mistakes in some b.c.'s)
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;% + 1 * rdcell;
            for k = 1:2*L
                dRdt((k-1)*2*W+ceil(W)+paramsSys.TgrPos) =  dRdt((k-1)*2*W+ceil(W)+paramsSys.TgrPos) + paramsSys.TgrInts;
            end
%             if paramsModel.CutOff ~= 0
%                 scale = 10^(-paramsModel.CutOff); % scale ~ sensitivity
%                 Ir = sparse(diag(R));
%                 TrsfrM = sparse(paramsSys.Trans * Ir - Ir * paramsSys.Trans);
% %                 TrsfrM(TrsfrM ~= 0) = TrsfrM(TrsfrM ~= 0) - 1./(TrsfrM(TrsfrM ~= 0)./paramsModel.CutOff  + 1./TrsfrM(TrsfrM ~= 0));
%                 TrsfrM(TrsfrM ~= 0) = TrsfrM(TrsfrM ~= 0) - 1./(TrsfrM(TrsfrM ~= 0)./scale  + 1./TrsfrM(TrsfrM ~= 0));
%                 dAdt = dAdt + Da * sum(TrsfrM, 2);
%                 dRdt = dRdt + Dr * sum(TrsfrM, 2);
%             else
            dAdt_Df = (A' * paramsSys.Q)';
            dAdt_Df((dAdt_Df ~= 0) &(abs(dAdt_Df)< paramsModel.CutOff)) = 0;
            dAdt = dAdt + Da * dAdt_Df; 
            
            dRdt_Df = (R' * paramsSys.Q)';
            dRdt_Df((dRdt_Df ~= 0) &(abs(dRdt_Df)< paramsModel.CutOff)) = 0;
            dRdt = dRdt + Dr * dRdt_Df; 
%             end

            % Simple Cutoff function (discontinuous function)
%             cutoffFunc = 'Ctype';
%             switch cutoffFunc
%                 case Atype
%                     TrsfrM((TrsfrM ~= 0) & (abs(TrsfrM)< 10^(-paramsModel.CutOff))) = 0;
%                 case Ctype
%                     TrsfrM = TrsfrM - 1./(TrsfrM./paramsModel.CutOff  + 1./TrsfrM);                    
% %                     TrsfrM(TrsfrM ~= 0) = TrsfrM(TrsfrM ~= 0) - 1/(TrsfrM(TrsfrM ~= 0)/paramsModel.CutOff  + 1/TrsfrM(TrsfrM ~= 0));
%             end

            

            %construct output dydt from dh and dp
            output(1 : numSpecies : end - 1) = dAdt;
            output(2 : numSpecies : end - 0) = dRdt;
 
  
            
        case 'preModel' %pre-equilibration model
            %break out y into molecular species (they are interleaved)
             % the fixed concentration of Hh to use for pre-equilibration
            A = y(1);
            R = y(2);

            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;
            
            %output
            output = [dAdt; dRdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;

 
        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
            % everything EXCEPT integrated signal q
%             q = 5 : numSpecies : numSpecies*N;   
            temp = 1 : numSpecies*N;
            output = temp; %look at everything except q
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2];
            
        case 'preModelIC' %specified initial conditions for preModel
            A0 = a_A/g_A;%a_p / c_p;
%             R0 = a_R/g_R;
%             tspan = [0 10000];
%             y0 = [A0 R0];
%             [t, y] = ode15s(@(t, y)preSol(t, y, paramsModel), tspan, y0, paramsModel);
%             output = [y(end,1), y(end,2)];
            
            
            tspan = [0 10000];
            y0 = A0;
            % y' = a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + ((g_A * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - g_A * y;
            [t, y] = ode45(@(t, y) a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + ((g_A * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - g_A * y, tspan, y0);
%             As = y(end)
%             Rs =  g_A * a_R * As/(g_R * a_A)
            output = [y(end), g_A * a_R * y(end)/(g_R * a_A)];
    end
end
% function dydt = preSol(t, y, paramsModel)
%     a_A = paramsModel.a_A;
%     a_R = paramsModel.a_R;
%     
%     g_A = paramsModel.g_A; 
%     g_R = paramsModel.g_R; 
%     
%     K_A = paramsModel.K_A;
%     K_R = paramsModel.K_R;
%     
%     K = paramsModel.K;    
%     H = paramsModel.H;
%     
%     dydt = [a_A * ((y(1)/K_A).^H + (1/K).^H)./((y(1)/K_A).^H + (y(2)/K_R).^H + 1 + (1/K).^H) - g_A * y(1), 
%         g_R * a_A * y(2) - g_A * a_R * y(1)];
% end
