%% Simulation for data in 3ABC
% Simulation platform for the SMRD paper 2C1D + RN + no noise

%%%%%%%% Kinetics_squ_Simu.m %%%%%%%%
% Model definition file
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = Kinetics_squ_SimuDDE(t, y, Z, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity
    N = paramsSys.N; % number of gridpoints
    L = paramsSys.L;
    W = paramsSys.W;
    flag = paramsSys.flag;

    a_A = paramsModel.a_A;
    a_R = paramsModel.a_R;
    
    g_A = paramsModel.g_A; 
    g_R = paramsModel.g_R; 
    
    K_A = paramsModel.K_A;
    K_R = paramsModel.K_R;
    
    K = paramsModel.K;    
    Dr = paramsModel.Dr;
    Da = paramsModel.Da;
    H = paramsModel.H;


    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*N, 1);            

            A = y(1 : numSpecies : end - 1);
            R = y(2 : numSpecies : end - 0);
            Ad = Z(1 : numSpecies : end - 1,1);
            Rd = Z(2 : numSpecies : end - 0,2);

            if paramsSys.taoA == 0
                Ad = A;
            end
            if paramsSys.taoR == 0
                Rd = R;
            end
            %equations for non-diffusible A and R
            dAdt = a_A * ((Ad/K_A).^H + (1/K).^H)./((Ad/K_A).^H + (Rd/K_R).^H + 1 + (1/K).^H) - g_A * A;
            dRdt = a_R * ((Ad/K_A).^H + (1/K).^H)./((Ad/K_A).^H + (Rd/K_R).^H + 1 + (1/K).^H) - g_R * R;
%             else
%                 dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;
%                 dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;
%             end
            dAdt_Df = (Da * A' * paramsSys.Q)';
            dAdt = dAdt + dAdt_Df; 
            dRdt_Df = (R' * paramsSys.Q)';
            dRdt_Df((dRdt_Df ~= 0) &(abs(dRdt_Df)< paramsModel.CutOff)) = 0;
            dRdt = dRdt + Dr * dRdt_Df;
            
            dRdt(1) = dRdt(1)+ 1;
%             if (t > 1) & (flag ==0)
%                 dRdt(1) = dRdt(1)+ 0.0001;
%                 paramsSys.flag = 1;
%             end

            output(1 : numSpecies : end - 1) = dAdt;
            output(2 : numSpecies : end - 0) = dRdt;
 
  
            
        case 'preModel' %pre-equilibration model
            A = y(1);
            R = y(2);
%             Rdelay = Z(:,2);

            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;
            
            %output
            output = [dAdt; dRdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;

 
        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
 
            temp = 1 : numSpecies*N;
            output = temp; 
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2];
            
        case 'preModelIC' %specified initial conditions for preModel
            A0 = a_A/g_A;

            tspan = [0 10000];
            y0 = A0;
            [t, y] = ode45(@(t, y) a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + ((g_A * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - g_A * y, tspan, y0);
            output = [y(end), g_A * a_R * y(end)/(g_R * a_A)];
    end
end
