%% Simulation for data in 3ABC
% Simulation platform for the SMRD paper 2C1D + RN + no noise

%%%%%%%% modelSim_squ_Simu.m %%%%%%%%
% Input arguments: 
%       modelHandle:        handle to model definition file
%       paramsModelRange:   structure with fields corresponding to model parameters.  Each field 
%                           contains a scalar if you want the parameter to be fixed, or a vector
%                           containing all values of the parameter you want to use
%       paramsSys:          structure containing parameters needed to describe model execution
%       returnEndpoints:    Return endpoints (1) or don't return endpoints (0)
%       returnResults:      Return results at all timepoints (1) or don't return results at all timepoints (0)
%       outfileName:        Name of file to which you want to save the results.  Use an empty string ('') to not save results to file
%
% Output arguments:
%       results:            Structure array containing simulation results for each parameter
%                           gridpoint.  Empty if returnResults == 0
%       endpoints:          Structure array containing endpoints of simulations for each parameter
%                           gridpoint.  Empty if returnEndpoints == 0
%       paramsModelGrid:    Structure array containing parameter values at each gridpoint.

%%
function results = modelSim_squ_SimuDDE(modelHandle, para, paramsModel, paramsSys, plotSimpEnd, plotSSAR, plotGIF, plotKymo)
    %% Setup for model execution    
    N = paramsSys.N; 
    L = paramsSys.L;
    W = paramsSys.W;
    numSpecies = paramsSys.numSpecies;

    paramsSys.taoR = para(1);
    paramsSys.taoA = para(2);

    
    %% run simulation
    disp('Running dynamic model')  
    % specificy Jacobian sparsity pattern to speed up solver. 
    S = [];%modelHandle([], [] ,[], 'jacobianSparsity', paramsModel, paramsSys); %get the Jacobian sparsity pattern from the model function 
    %start timer
    tic
        
    %initialize results structure
    val0 = cell(1,1);
    localResults = struct('tPre', val0, 'y0Pre', val0, 'yPre', val0, 'outFlagPre', val0, 't', val0, 'y', val0, 'outFlag', val0);

    %set up model handle and solver options for pre-equilibration
    modelHandlePredde23 = @(t,y,Z) modelHandle(t, y, Z, 'preModel', paramsModel, paramsSys); %for Ptch preq-equilibration
    optionsdde23Pre = ddeset('Events', [], 'InitialStep', 1e-4, 'MaxStep', 1e-2);

    %set up model handle, events handle, and solver options
    modelHandledde23 = @(t,y,Z) modelHandle(t, y, Z, 'mainModel', paramsModel, paramsSys);
    if ~isempty(S)
        optionsdde23 = ddeset('JPattern', S, 'InitialStep', 1e-4, 'MaxStep', 1e-2);%, 'RelTol',1e-8, 'AbsTol', 1e-9);
    else
        optionsdde23 = ddeset('InitialStep', 1e-4, 'MaxStep', 1e-2);
    end

    %% pre-equilibrate species in absence of morphogen diffusion
    species2EquilPre = modelHandle([], [],[], 'preSpeciesSs', paramsModel, paramsSys); %species to test for steady-stateness
    tSpan = paramsSys.tSpanPre;
    maxT = paramsSys.maxTPre;
    y0Pre = modelHandle([], [], [],'preModelIC', paramsModel, paramsSys); %initial conditions for pre-equilibration
    localResults.y0Pre = y0Pre;
    [tPre, yPre, outFlagPre] = dde23TillSs(modelHandlePredde23, y0Pre, species2EquilPre, tSpan, maxT, optionsdde23Pre, paramsSys);
    localResults.tPre = tPre(end); %last timepoint
    localResults.yPre = yPre(end,:); %store only last timepoint
    localResults.outFlagPre = outFlagPre;

    %run solver with y0 equal to pre-equilibrated y until a steady state is reached
    %set y0 based on output of pre-equilibration
    species2Equil = modelHandle([], [], [],'speciesSs', paramsModel, paramsSys); %species to test for steady-stateness
    y0Main = zeros(numSpecies*N,1); % initial conditions for mainModel

    %% Add noise after pre-equilibration and run main simulation
    for j = 1:N % populate initial conditions
        y0Main((j-1)*numSpecies+1) = yPre(end,1);
        y0Main(j*numSpecies) = yPre(end,2);
    end
%     switch paramsSys.TgrType 
%         case 'RN' % Random noise to the A and R profile
%             for j = 1:N 
%                 y0Main(j*numSpecies) = y0Main(j*numSpecies) * ( 1 + 0.01 * (paramsSys.TgrInts+1) * rand());
%             end
%         case 'SP' % Activators on one point 
%             sp = ceil(N/2);
%             y0Main(sp*numSpecies) = y0Main(sp*numSpecies) * (1+2^(-paramsSys.TgrInts));
%         case 'LA' % Activators on one line 
%             for k = 1:2*L
%                 y0Main(((k-1)*2*W+paramsSys.TgrPos)*numSpecies) =  y0Main(((k-1)*2*W+paramsSys.TgrPos)*numSpecies) * (1+2^(-paramsSys.TgrInts));
%             end
%     end
    
    tSpan = paramsSys.tSpan;
    maxT = paramsSys.maxT;
    y0Tem = y0Main;
    localResults.t = [];
    localResults.y = [];
    for rst = 1:paramsSys.resetCycle
        [localResults.tTem, localResults.yTem, localResults.outFlag] = dde23TillSs(modelHandledde23, y0Tem, species2Equil, tSpan, maxT, optionsdde23, paramsSys);
        localResults.t = [localResults.t; localResults.tTem];
        localResults.y = [localResults.y; localResults.yTem];
        y0Tem = localResults.yTem(end,:);
    end
    
    %% record and plot
%     filenameData = sprintf('Figdata/Fig_delay%08.4f_%dx%d', ... 
%                     0, 2*paramsSys.L, 2*paramsSys.W); 
    filenameData = sprintf('Figdata/Fig_delayR%08.4f_delayA%08.4f_%dx%d', ... 
                    paramsSys.taoR, paramsSys.taoA, 2*paramsSys.L, 2*paramsSys.W);  
    for k = 0:1:paramsSys.N-1
        i = floor(k/(2*paramsSys.W))+1;
        j = mod(k, 2*paramsSys.W)+1;
        localResults.Za(i,j,:) = localResults.y(:,2*k+1);
        localResults.Zr(i,j,:) = localResults.y(:,2*k+2);
    end
    disp('Recording results')
    results = localResults;
    filename_Mat = strcat(filenameData,'.mat');    
    save(filename_Mat,'results')

    if plotSimpEnd == 1
        plotSimpleEndpoint(results, filenameData);
    end
    if plotSSAR == 1
        plotEndpointARsurf(results, filenameData);
    end
    if plotGIF == 1
        makeGIFplot(results, filenameData);
    end
    if plotKymo == 1
        plotKymogragh(results, filenameData);
    end

    toc %stop timer
    

end

%% dde23TillSs
% simulates the model using dde23 until steady state is reached
function [t, y, outFlag] = dde23TillSs(modelHandle, y0, species2Equil, tSpan, maxT, options, paramsSys)
% tSpan and maxT override anything in paramsSys!
%outFlag
%   1 -- reached steady state with in tSpan
%   2 -- did not reach steady state and hit maxTime
%   3 -- reached steady state with in tSpan
%   4 -- error was generated    

    isSs = 0;
    isMaxTime = 0;
    y0Temp = y0; %initial conditions
    t = 0;
    y(1,:) = y0;
    outFlag = 0;
    preRun = 0; %pre-run will be 1 if sim runs before min(tSpan); see below
    lags = [paramsSys.taoA, paramsSys.taoR];
    lags(lags == 0) =1;
%     his = @(t)history(t,y0);
    %if tSpan doesn't start with 0, run sim until minimum of tSpan
    if min(tSpan) ~= 0
        try
            sol = dde23(modelHandle, lags, y0Temp, [0 min(tSpan)], options);
            tTemp = sol.x';
            yTemp = sol.y';
            y0Temp = yTemp(end,:);
            y(1,:) = y0Temp;
            t = tTemp(end);
            preRun = 1;
        catch exception
            exception.getReport
            outFlag = 4;
        end
    end
    
    while ~isSs && ~isMaxTime && outFlag ~= 4
        try
            if preRun == 0
                sol = dde23(modelHandle, lags, y0Temp, tSpan + max(t), options);
                tTemp = sol.x';
                yTemp = sol.y'; 
            else
                sol = dde23(modelHandle, lags, y0Temp, tSpan, options); 
                tTemp = sol.x';
                yTemp = sol.y';
            end
            preRun = 0;
        catch exception
            exception.getReport
            outFlag = 4;
        end
        
        try
            t = cat(1, t, tTemp(2:end)); %add tTemp to end of t{i}
            y = cat(1, y, yTemp(2:end,:)); %add yTemp to the end of y{i}
        catch exception
            exception.getReport
            outFlag = 4;
        end

        if outFlag ~= 4 %no error
            flag = isOutputSteadyState(t, y, species2Equil, paramsSys);
            if flag == 1 %steady state reached
                isSs = 1;
                if max(t) <= max(tSpan)%maxT
                    isMaxTime = 0;
                    outFlag = 1;
                elseif max(t) <= maxT
                    isMaxTime = 0;
                    outFlag = 3;
                end
            else  %steady state not reached
                if max(t) < maxT
                    y0Temp = y(end,:); %set y0Temp for next run equal to the ending y
                else
                    isMaxTime = 1;
                    outFlag = 2;
                end
            end
        end
    end
    
    outFlag;
end
% function y0t = history(t, y0)
%     y0t = y0;
% %     y0t(2) = y0t(2)+0.0001*dirac(t);
% end
%% isOutputSteadyState
function flag = isOutputSteadyState(t, y, species2Equil, paramsSys) 
%At steady state if (a) biggest magnitude FRACTIONAL change between any element of y over the 
%last sSLookbackSteps and the first lookback step is less than steadyStateTol, (b) at least sSLookbackSteps have run, and (c) t > tMin
    numSteps = size(y,1);
    if (numSteps >= paramsSys.minTimeSteps) && (t(end) >= paramsSys.tMin)
        lookbackTRange = numSteps - paramsSys.sSLookbackSteps + 1 : numSteps;
        lookback = y(lookbackTRange, :); 
        
        for j = species2Equil
            lookbackFracDiff(:, j) = lookback(:,j) / lookback(1,j) - 1;
        end
        
        maxDist = max(max(abs(lookbackFracDiff))); 
        if maxDist < paramsSys.steadyStateTol
            flag = 1;
        else
            flag = 0;
        end
    else
        flag = 0;
    end
end
%% plotSimpleEndpoint
function plotSimpleEndpoint(results, filenameData)
    disp('Recording endpoints')
    figure;
    ZaEnd = results.Za(:,:,end);
    image(ZaEnd);
    colorbar;
    if results.outFlagPre ~= 2 && results.outFlagPre ~= 4
        switch results.outFlag
            case 1
                colormap(parula);
            case 2
                colormap(spring);            
            case 3
                colormap(summer);
            case 4
                colormap(gray);            
        end
    else
        colormap(copper);
    end
    filename_A = strcat(filenameData,'-A-lite.fig');
    saveas(gcf,filename_A);
    close;
end
%% plotEndpointARsurf
function plotEndpointARsurf(results, filenameData)
    disp('Recording endpoints')
    figure;
    ZaEnd = results.Za(:,:,end);
    ZrEnd = results.Zr(:,:,end);
    surf(ZrEnd);
    filename_R = strcat(filenameData,'-R.fig');
    saveas(gcf,filename_R);
    surf(ZaEnd);
    filename_A = strcat(filenameData,'-A.fig');
    saveas(gcf,filename_A);
    close;
end
%% make a GIF plot
function makeGIFplot(results, filenameData)
    % Plot gif
    h = figure;
    axis tight
    set(gca,'nextplot','replacechildren','visible','off')
    frame = getframe(h);
    im = frame2im(frame); 
    [imind,cm] = rgb2ind(im,256);
    filename_MV = strcat(filenameData,'_MV-A.gif');
    imwrite(imind,cm,filename_MV,'gif', 'Loopcount',inf); 

    for t = 1:size(results.Za,3)
        imagesc(results.Za(:,:,t));
        frame = getframe(h);
        im = frame2im(frame); 
        colorbar;
        [imind,cm] = rgb2ind(im,256);
        imwrite(imind,cm,filename_MV,'gif','WriteMdde','append'); 
    end        

end
%% plot a kymogragh 
function plotKymogragh(results, filenameData)
    h = figure;
    AW_mean = mean(results.Za,1);
    RW_mean = mean(results.Zr,1);
    AL_mean = mean(results.Za,2);
    RL_mean = mean(results.Zr,2);
    kymoplot = 'W';
    switch kymoplot
        case 'LW'
            subplot(2,2,1)
            imagesc(squeeze(AW_mean(1,:,:))')
            title('[A] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(2,2,2)
            imagesc(squeeze(RW_mean(1,:,:))')
            title('[R] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')   
            subplot(2,2,3)
            imagesc(squeeze(AL_mean(:,1,:))')
            title('[A] along L')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(2,2,4)
            imagesc(squeeze(RL_mean(:,1,:))')
            title('[R] along L')
            xlabel('cell position')
            ylabel('time (a.u.)') 
        case 'W'
            subplot(1,2,1)
            imagesc(squeeze(AW_mean(1,:,:))')
            title('[A] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(1,2,2)
            imagesc(squeeze(RW_mean(1,:,:))')
            title('[R] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')   
    end
    filename_KM = strcat(filenameData,'_ML.fig');
    saveas(h, filename_KM)
end
