%% Simulation for data in 3ABC
% Simulation platform for the SMRD paper 2C1D + RN + no noise

%%%%%%%% Kinetics_squ_Simu.m %%%%%%%%
% Model definition file
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = Kinetics_hex_Simu(t, y, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity
    N = paramsSys.N; % number of gridpoints
    L = paramsSys.L;
    W = paramsSys.W;

    a_A = paramsModel.a_A;
    a_R = paramsModel.a_R;
    
    g_A = paramsModel.g_A; 
    g_R = paramsModel.g_R; 
    
    K_A = paramsModel.K_A;
    K_R = paramsModel.K_R;
    
    K = paramsModel.K;    
    Dr = paramsModel.Dr;
    Da = paramsModel.Da;
    H = paramsModel.H;


    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*N, 1);            

            A = y(1 : numSpecies : end - 1);
            R = y(2 : numSpecies : end - 0);


            %equations for non-diffusible A and R
            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;

%             dAdt_Df = (Da * A' * paramsSys.Q)';
%             dAdt = dAdt + dAdt_Df; 
%             dRdt_Df = (R' * paramsSys.Q)';
%             dRdt_Df((dRdt_Df ~= 0) &(abs(dRdt_Df)< paramsModel.CutOff)) = 0;
%             dRdt = dRdt + Dr * dRdt_Df;
            
            for k = 0:1:N-1
                i = floor(k/(2*L));
                j = mod(k, 2*L);
                if mod(i, 2) == 1
                    %TODO
                    up = i * 2*L + mod(j-1, 2*L);
                    dn = i * 2*L + mod(j+1, 2*L);
                    lu = mod(i-1, 2*W)* 2*L + j;
                    ld = mod(i-1, 2*W)* 2*L + mod(j+1, 2*L);
                    ru = mod(i+1, 2*W)* 2*L + j;
                    rd = mod(i+1, 2*W)* 2*L + mod(j+1, 2*L);
%                         print ([up, dn, lu, ld, ru, rd])
                else
                    %TODO
                    up = i * 2*L + mod(j-1, 2*L);
                    dn = i * 2*L + mod(j+1, 2*L);
                    lu = mod(i-1, 2*W)* 2*L + mod(j-1, 2*L);
                    ld = mod(i-1, 2*W)* 2*L + j;
                    ru = mod(i+1, 2*W)* 2*L + mod(j-1, 2*L);
                    rd = mod(i+1, 2*W)* 2*L + j;
%                         [up, dn, lu, ld, ru, rd]
                end
                dRdt(k+1) = dRdt(k+1) + Dr * (R(up+1) + R(dn+1)+ R(lu+1) + R(ld+1) + R(ru+1) + R(rd+1) - 6* R(k+1));
            end

            output(1 : numSpecies : end - 1) = dAdt;
            output(2 : numSpecies : end - 0) = dRdt;
 
  
            
        case 'preModel' %pre-equilibration model
            A = y(1);
            R = y(2);

            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A;
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;
            
            %output
            output = [dAdt; dRdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;

 
        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
 
            temp = 1 : numSpecies*N;
            output = temp; 
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2];
            
        case 'preModelIC' %specified initial conditions for preModel
            A0 = a_A/g_A;

            tspan = [0 10000];
            y0 = A0;
            [t, y] = ode45(@(t, y) a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + ((g_A * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - g_A * y, tspan, y0);
            output = [y(end), g_A * a_R * y(end)/(g_R * a_A)];
    end
end
