%% Simulation for data in 6E
% Simulation platform for the SMRD paper 2C1D(growth-coupled) + LA + with noise

%%%%%%%% Kinetics_squ_SMRD_ApiDom.m %%%%%%%%
% Model definition file
% Simulates model for N gridpoints, using a vector of N*numSpecies
%       E.g., for species A, R, and TD (numSpecies  == 3), and N == 3
%             y = [h1, p1, c1, r1, q1, h2, p2, c2, r2, q2, h3, p3, c3, r3, q3], where h1 =
%             concentration of h at cell 1, h2 = concentration of h at cell 2, etc
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = Kinetics_squ_SMRD_GroCou(t, y, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity

    Nc = paramsSys.Nc; % number of gridpoints
    w = 2* paramsSys.W;
    a_A = paramsModel.a_A;
    a_R = paramsModel.a_R;
    
    g_A = paramsModel.g_A; 
    g_R = paramsModel.g_R; 
    
    K_A = paramsModel.K_A;
    K_R = paramsModel.K_R;
    
    K = paramsModel.K;    
    Dr = paramsModel.Dr;
    H = paramsModel.H;

    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*Nc, 1);            
            A = y(1 : numSpecies : end - 1);
            R = y(2 : numSpecies : end - 0);
            Ap = A(1 : end - w); % patterning cells
            Rp = R(1 : end - w);
            As = A(end - w + 1 : end); % sender-progenitor cells
            Rs = R(end - w + 1 : end);

            %equations for non-diffusible A and R
            c = interp1(paramsSys.tDyNois', paramsSys.yDyNois', t);
            c = c(1:Nc-w);
%             c = ones(1, Nc-w);

            dAdt(1:Nc-w,1) = a_A * c' .* ((Ap/K_A).^H + (1/K).^H)./((Ap/K_A).^H + (Rp/K_R).^H + 1 + (1/K).^H) - g_A * Ap; 
            dAdt(Nc-w+1:Nc,1) = 0; 
            dRdt(1:Nc-w,1) = a_R * c' .* ((Ap/K_A).^H + (1/K).^H)./((Ap/K_A).^H + (Rp/K_R).^H + 1 + (1/K).^H) - g_R * Rp;% + 1 * rdcell;
            dRdt(Nc-w+1:Nc,1) = 0.5 * a_R - g_R * Rs;

            dRdt_Df = (R' * paramsSys.Q)';
            dRdt_Df((dRdt_Df ~= 0) &(abs(dRdt_Df)< paramsModel.CutOff)) = 0;
            dRdt = dRdt + Dr * dRdt_Df; 

            output(1 : numSpecies : end - 1) = dAdt;
            output(2 : numSpecies : end - 0) = dRdt;
  
        case 'preModel' %pre-equilibration model

            A = y(1);
            R = y(2);

            dAdt = 0;
            dRdt = 0.5 * a_R - g_R * R;
            
            %output
            output = [dAdt; dRdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;

        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
   
            temp = 1 : numSpecies*Nc;
            output = temp; %look at everything except q
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2];
            
        case 'preModelIC' %specified initial conditions for preModel
            A0 = a_A/g_A;
            tspan = [0 10000];
            y0 = A0;
            [t, y] = ode45(@(t, y) a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + ((g_A * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - g_A * y, tspan, y0);
            output = [0, 0.5 * a_R /g_R];
    end
end
