%% Simulation for data in 5F
% Simulation platform for the SMRD paper 2C1D(inhibition-release) + LA + with noise

%%%%%%%% Turing_191221_squ.m %%%%%%%%
% Input arguments: 
%       t:                  time (scalar)
%       y:                  vector containing species concentrations
%       outType:            the type of output you want to be returned
%                           'mainModel':        the model itself, to be run after pre-equilibration
%                           'preModel':         pre-equilibration model.  SPATIALLY HOMOGENEOUS
%                           'perception':       function that calculates perception at each gridpoint
%                           'speciesSS':        vector containing indices of species to be tested for steady-stateness in mainModel
%                           'preSpeciesSs':     vector containing indices of species to be tested for steady-stateness in preModel
%                           'preModelIC':       initial conditions for preModel
%                           'jacobianSparsity': OPTIONAL. returns sparsity pattern of Jacobian for
%                                               mainModel.  If you don't want to type it out, set
%                                               to empty vector ([])
%       paramsModel:        structure containing model parameters
%       paramsSys:          structure containing parameters needed to describe model execution
%
% Output arguments:
%       output:             depends on "outType"


function output = Kinetics_squ_SMRD_InhRel(t, y, outType, paramsModel, paramsSys)  

    numSpecies = paramsSys.numSpecies; % number of species in model
    
    %put params into separate variables, for clarity

    N = paramsSys.N; % number of gridpoints
    L = paramsSys.L;
    W = paramsSys.W;
    a_A = paramsModel.a_A;
    a_R = paramsModel.a_R;
    
    g_A = paramsModel.g_A; 
    g_R = paramsModel.g_R;
    
    K_A = paramsModel.K_A;
    K_R = paramsModel.K_R;
    K_T = paramsModel.K_T;
    K_P = paramsModel.K_P;
    
    K = paramsModel.K;    
    Dr = paramsModel.Dr;
    H = paramsModel.H;

    switch outType 
        case 'mainModel' %full model
            output = zeros(numSpecies*N, 1);            
            A = y(1 : numSpecies : end - 2);
            R = y(2 : numSpecies : end - 1);
            P = y(3 : numSpecies : end - 0);

            %equations for non-diffusible A and R
            c = interp1(paramsSys.tDyNois', paramsSys.yDyNois', t);
%             c = ones(1, N);

            dAdt = a_A * c' .* ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A - 3 * (1 - P) .* A; 
            dRdt = a_R * c' .* ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;% + 1 * rdcell;
            dPdt = P.^2./(P.^2+K_P.^2) + 0.01 * (abs(R-K_T) + (R-K_T))./(abs(R-K_T)+eps) - P;

            dRdt_Df = (R' * paramsSys.Q)';
            dRdt_Df((dRdt_Df ~= 0) &(abs(dRdt_Df)< paramsModel.CutOff)) = 0;
            dRdt = dRdt + Dr * dRdt_Df; 

            output(1 : numSpecies : end - 2) = dAdt;
            output(2 : numSpecies : end - 1) = dRdt;
            output(3 : numSpecies : end - 0) = dPdt;
     
        case 'preModel' %pre-equilibration model

            A = y(1);
            R = y(2);
            P = y(3);

            dAdt = a_A * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_A * A - 3*(1-0).*A;
            dRdt = a_R * ((A/K_A).^H + (1/K).^H)./((A/K_A).^H + (R/K_R).^H + 1 + (1/K).^H) - g_R * R;
            dPdt = P.^2./(P.^2+K_P.^2) + 0.01 * (abs(R-K_T) + (R-K_T))./(abs(R-K_T)+eps) - P;

            %output
            output = [dAdt; dRdt; dPdt];
                 
        case 'jacobianSparsity' % sparsity pattern of model Jacobian

            output = paramsSys.S;


        case 'speciesSs' % indices of species to be tested for steady-stateness in main model
  
            temp = 1 : numSpecies*N;
            output = temp; %look at everything except q
            
        case 'preSpeciesSs' % indices of species to be tested for steady-stateness in preModel
            %  [A, R] 
            output = [1, 2, 3];
            
        case 'preModelIC' %specified initial conditions for preModel            
            tspan = [0 10000];
            y0 = 0.1;
            [t, y] = ode45(@(t, y) a_A * ((y./K_A).^H + (1/K).^H)./((y./K_A).^H + (((g_A + 3) * a_R * y./(g_R * a_A))./K_R).^H + 1 + (1/K).^H) - (g_A + 3) * y, tspan, y0);
            As = y(end);
            Rs =  (g_A + 3) * a_R * As/(g_R * a_A);

            output = [As, Rs, 0];
    end
end
