%% Simulation for data in 5F
% Simulation platform for the SMRD paper 2C1D(inhibition-release) + LA + with noise

%%%%%%%% modelSim_squ_SMRD_preLock.m %%%%%%%%
% preLock + Noise (0716)
% Edit from 200504_PreLocked_2C1D_propagation/OUSP RunMe_squ_2C1Dftpal and 0715_SimuNoise
% Simulation platform for the SMRD paper 2C1D (preLock) + noise 
% Input arguments: 
%       modelHandle:        handle to model definition file
%       paramsModelRange:   structure with fields corresponding to model parameters.  Each field 
%                           contains a scalar if you want the parameter to be fixed, or a vector
%                           containing all values of the parameter you want to use
%       paramsSys:          structure containing parameters needed to describe model execution
%       returnEndpoints:    Return endpoints (1) or don't return endpoints (0)
%       returnResults:      Return results at all timepoints (1) or don't return results at all timepoints (0)
%       outfileName:        Name of file to which you want to save the results.  Use an empty string ('') to not save results to file
%
% Output arguments:
%       results:            Structure array containing simulation results for each parameter
%                           gridpoint.  Empty if returnResults == 0
%       endpoints:          Structure array containing endpoints of simulations for each parameter
%                           gridpoint.  Empty if returnEndpoints == 0
%       paramsModelGrid:    Structure array containing parameter values at each gridpoint.

%%
function results = modelSim_squ_SMRD_InhRel(modelHandle, para, paramsModel, paramsSys, plotSimpEnd, plotSSAR, plotGIF, plotKymo)
    %% Setup for model execution    
    % get some values from input data structures for convenience
    N = paramsSys.N; %shorter name for convenience
    L = paramsSys.L;
    W = paramsSys.W;
    numSpecies = paramsSys.numSpecies;

    sig = para(1);
    eta = para(2);
    trailcont = para(3);

    %% run simulation
    disp('Running dynamic model')  
    % specificy Jacobian sparsity pattern to speed up solver. 
    S = modelHandle([], [] , 'jacobianSparsity', paramsModel, paramsSys); %get the Jacobian sparsity pattern from the model function 
    %start timer
    tic
        
    %initialize results structure
    val0 = cell(1,1);
    localResults = struct('paramsModel', paramsModel, 'paramsSys', paramsSys,'tPre', val0, ...
        'y0Pre', val0, 'yPre', val0, 'outFlagPre', val0, 't', val0, 'y', val0, 'outFlag', val0, ...
        'DyNoise1', val0, 'DyNoise2', val0);
    %% Intrinsic noise pre-culculation with OU method
    mu = 1;

    [paramsSys.tDyNois, paramsSys.yDyNois] = noiseGenerator_squ(paramsSys, mu, sig, eta, paramsSys.rdcell);
    localResults.DyNoise1 = paramsSys.yDyNois;
 
    %set up model handle and solver options for pre-equilibration
    modelHandlePreOde15s = @(t,y) modelHandle(t, y, 'preModel', paramsModel, paramsSys); %for Ptch preq-equilibration
    optionsOde15sPre = odeset('Events', [], 'InitialStep', 1e-4, 'MaxStep', 1e-2);

    %set up model handle, events handle, and solver options
    modelHandleOde15s = @(t,y) modelHandle(t, y, 'mainModel', paramsModel, paramsSys);
    if ~isempty(S)
        optionsOde15s = odeset('JPattern', S, 'InitialStep', 1e-4, 'MaxStep', 1e-2);
    else
        optionsOde15s = odeset('InitialStep', 1e-4, 'MaxStep', 1e-2);
    end

    %% pre-equilibrate species in absence of morphogen diffusion
    species2EquilPre = modelHandle([], [], 'preSpeciesSs', paramsModel, paramsSys); %species to test for steady-stateness
    tSpan = paramsSys.tSpanPre;
    maxT = paramsSys.maxTPre;
    y0Pre = modelHandle([], [], 'preModelIC', paramsModel, paramsSys); %initial conditions for pre-equilibration
    localResults.y0Pre = y0Pre;
    [tPre, yPre, outFlagPre] = ode15sTillSs(modelHandlePreOde15s, y0Pre, species2EquilPre, tSpan, maxT, optionsOde15sPre, paramsSys);
    localResults.tPre = tPre(end); %last timepoint
    localResults.yPre = yPre(end,:); %store only last timepoint
    localResults.outFlagPre = outFlagPre;

    %run solver with y0 equal to pre-equilibrated y until a steady state is reached
    %set y0 based on output of pre-equilibration
    species2Equil = modelHandle([], [], 'speciesSs', paramsModel, paramsSys); %species to test for steady-stateness
    y0Main = zeros(numSpecies*N,1); % initial conditions for mainModel

    %% Add noise after pre-equilibration and run main simulation
    for j = 1:N % populate initial conditions
        y0Main((j-1)*numSpecies+1) = yPre(end,1);
        y0Main((j-1)*numSpecies+2) = yPre(end,2);
        y0Main(j*numSpecies) = yPre(end,3);
    end
    switch paramsSys.TgrType 
        case 'RN' % Random noise to the A and R profile
            for j = 1:N % populate initial conditions
                y0Main((j-1)*numSpecies+1) = y0Main((j-1)*numSpecies+1) * ( 1 + 0.01 * paramsSys.TgrInts * paramsSys.rdcell(j,1));
            end
        case 'SP' % Activators on one point 
            sp = fix((N/2))+paramsSys.TgrPos;
            y0Main(sp*numSpecies) = y0Main(sp*numSpecies) * (1+2^(-paramsSys.TgrInts));
        case 'LA' % Activators on one line 
            for k = 1:2*L
                y0Main(((k-1)*2*W+paramsSys.TgrPos)*numSpecies+2) =  y0Main(((k-1)*2*W+paramsSys.TgrPos)*numSpecies+2) * (10+2^(-paramsSys.TgrInts));
            end
    end
    
    tSpan = paramsSys.tSpan;
    maxT = paramsSys.maxT;
    y0Tem = y0Main;
    localResults.t = [];
    localResults.y = [];
    for rst = 1:paramsSys.resetCycle
        [localResults.tTem, localResults.yTem, localResults.outFlag] = ode15sTillSs(modelHandleOde15s, y0Tem, species2Equil, tSpan, maxT, optionsOde15s, paramsSys);
        localResults.t = [localResults.t; localResults.tTem];
        localResults.y = [localResults.y; localResults.yTem];
        y0Tem = localResults.yTem(end,:);
    end
    
    %% record and plot
    filenameData = sprintf('IRdata/Dr%.2f_t%d_%s_%s_sig%.8feta%.8f_%dx%d_trail%d', ... 
                    paramsModel.Dr, paramsSys.tSpanMax, ...
                    paramsSys.bdryType, paramsSys.TgrType, sig, eta, ...
                    2*paramsSys.L, 2*paramsSys.W, trailcont);  
    for k = 0:1:paramsSys.N-1
        i = floor(k/(2*paramsSys.W))+1;
        j = mod(k, 2*paramsSys.W)+1;
        localResults.Za(i,j,:) = localResults.y(:,numSpecies*k+1);
        localResults.Zr(i,j,:) = localResults.y(:,numSpecies*k+2);
        localResults.Zp(i,j,:) = localResults.y(:,numSpecies*k+3);
    end
    disp('Recording results')
    results = localResults;
    filename_Mat = strcat(filenameData,'.mat');    
    save(filename_Mat,'results')

    if plotSimpEnd == 1
        plotSimpleEndpoint(results, filenameData);
    end
    if plotSSAR == 1
        plotEndpointARsurf(results, filenameData);
    end
    if plotGIF == 1
        makeGIFplot(results, filenameData);
    end
    if plotKymo == 1
        plotKymogragh(results, filenameData);
    end

    toc %stop timer
    

end

%% ode15sTillSs
% simulates the model using ode15s until steady state is reached
function [t, y, outFlag] = ode15sTillSs(modelHandle, y0, species2Equil, tSpan, maxT, options, paramsSys)
% tSpan and maxT override anything in paramsSys!
%outFlag
%   1 -- reached steady state with in tSpan
%   2 -- did not reach steady state and hit maxTime
%   3 -- reached steady state with in tSpan
%   4 -- error was generated    

    isSs = 0;
    isMaxTime = 0;
    y0Temp = y0; %initial conditions
    t = 0;
    y(1,:) = y0;
    outFlag = 0;
    preRun = 0; %pre-run will be 1 if sim runs before min(tSpan); see below
    
    %if tSpan doesn't start with 0, run sim until minimum of tSpan
    if min(tSpan) ~= 0
        try
            [tTemp, yTemp] = ode15s(modelHandle, [0 min(tSpan)], y0Temp, options);        
            y0Temp = yTemp(end,:);
            y(1,:) = y0Temp;
            t = tTemp(end);
            preRun = 1;
        catch exception
            exception.getReport
            outFlag = 4;
        end
    end
    
    while ~isSs && ~isMaxTime && outFlag ~= 4
        %for testing saving only a few timepts
        try
            if preRun == 0
                [tTemp, yTemp] = ode15s(modelHandle, tSpan + max(t), y0Temp, options);        
            else
                [tTemp, yTemp] = ode15s(modelHandle, tSpan, y0Temp, options); 
            end
            preRun = 0;
        catch exception
            exception.getReport
            outFlag = 4;
        end
        
        try
            t = cat(1, t, tTemp(2:end)); %add tTemp to end of t{i}
            y = cat(1, y, yTemp(2:end,:)); %add yTemp to the end of y{i}
        catch exception
            exception.getReport
            outFlag = 4;
        end

        if outFlag ~= 4 %no error
            flag = isOutputSteadyState(t, y, species2Equil, paramsSys);
            if flag == 1 %steady state reached
                isSs = 1;
                if max(t) <= max(tSpan)%maxT
                    isMaxTime = 0;
                    outFlag = 1;
                elseif max(t) <= maxT
                    isMaxTime = 0;
                    outFlag = 3;
                end
            else  %steady state not reached
                if max(t) < maxT
                    y0Temp = y(end,:); %set y0Temp for next run equal to the ending y
                else
                    isMaxTime = 1;
                    outFlag = 2;
                end
            end
        end
    end
    outFlag;
end
%% isOutputSteadyState
function flag = isOutputSteadyState(t, y, species2Equil, paramsSys) 
%At steady state if (a) biggest magnitude FRACTIONAL change between any element of y over the 
%last sSLookbackSteps and the first lookback step is less than steadyStateTol, (b) at least sSLookbackSteps have run, and (c) t > tMin
    numSteps = size(y,1);
    if (numSteps >= paramsSys.minTimeSteps) && (t(end) >= paramsSys.tMin)
        lookbackTRange = numSteps - paramsSys.sSLookbackSteps + 1 : numSteps;
        lookback = y(lookbackTRange, :); 

        for j = species2Equil
            lookbackFracDiff(:, j) = lookback(:,j) / lookback(1,j) - 1;
        end
   
        maxDist = max(max(abs(lookbackFracDiff))); 
        if maxDist < paramsSys.steadyStateTol
            flag = 1;
        else
            flag = 0;
        end
    else
        flag = 0;
    end
end
%% plotSimpleEndpoint
function plotSimpleEndpoint(results, filenameData)
    disp('Recording endpoints')
    figure;
    ZaEnd = results.Za(:,:,end);
    image(ZaEnd);
    colorbar;
    if results.outFlagPre ~= 2 && results.outFlagPre ~= 4
        switch results.outFlag
            case 1
                colormap(parula);
            case 2
                colormap(spring);            
            case 3
                colormap(summer);
            case 4
                colormap(gray);            
        end
    else
        colormap(copper);
    end
    filename_A = strcat(filenameData,'-A-lite.fig');
    saveas(gcf,filename_A);
    close;
end
%% plotEndpointARsurf
function plotEndpointARsurf(results, filenameData)
    disp('Recording endpoints')
    figure;
    ZaEnd = results.Za(:,:,end);
    ZrEnd = results.Zr(:,:,end);
    ZpEnd = results.Zp(:,:,end);
    surf(ZpEnd);
    filename_P = strcat(filenameData,'-P.fig');
    saveas(gcf,filename_P);
    surf(ZrEnd);
    filename_R = strcat(filenameData,'-R.fig');
    saveas(gcf,filename_R);
    surf(ZaEnd);
    filename_A = strcat(filenameData,'-A.fig');
    saveas(gcf,filename_A);
    close;
end
%% make a GIF plot
function makeGIFplot(results, filenameData)
    % Plot gif
    h = figure;
    axis tight
    set(gca,'nextplot','replacechildren','visible','off')
    frame = getframe(h);
    im = frame2im(frame); 
    [imind,cm] = rgb2ind(im,256);
    filename_MV = strcat(filenameData,'_MV-A.gif');
    imwrite(imind,cm,filename_MV,'gif', 'Loopcount',inf); 
    colorbar;
    caxis manual;
    caxis([0 1]); 

    allA = results.y(:,1:2:end);
    mx = max(allA(:));
    norm_allA = allA./mx;
    size(norm_allA);
    for t = 1:size(results.Za,3)
        imagesc(results.Za(:,:,t));
        frame = getframe(h);
        im = frame2im(frame); 
        [imind,cm] = rgb2ind(im,256);
        imwrite(imind,cm,filename_MV,'gif','WriteMode','append'); 
    end        
end
%% plot a kymogragh 
function plotKymogragh(results, filenameData)
    h = figure;
    AW_mean = mean(results.Za,1);
    RW_mean = mean(results.Zr,1);
    AL_mean = mean(results.Za,2);
    RL_mean = mean(results.Zr,2);
    kymoplot = 'W';
    switch kymoplot
        case 'LW'
            subplot(2,2,1)
            imagesc(squeeze(AW_mean(1,:,:))')
            title('[A] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(2,2,2)
            imagesc(squeeze(RW_mean(1,:,:))')
            title('[R] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')   
            subplot(2,2,3)
            imagesc(squeeze(AL_mean(:,1,:))')
            title('[A] along L')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(2,2,4)
            imagesc(squeeze(RL_mean(:,1,:))')
            title('[R] along L')
            xlabel('cell position')
            ylabel('time (a.u.)') 
        case 'W'
            subplot(1,2,1)
            imagesc(squeeze(AW_mean(1,:,:))')
            title('[A] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')
            subplot(1,2,2)
            imagesc(squeeze(RW_mean(1,:,:))')
            title('[R] along W')
            xlabel('cell position')
            ylabel('time (a.u.)')   
    end
    filename_KM = strcat(filenameData,'_ML.fig');
    saveas(h, filename_KM)
end
